;Filename: UrtCfg.s
;Part    : ADuC703x.
;Created : 14/04/2006
;Author  : Patrick Noonan (Analog Device Inc.)
;This file is in beta sampling.

;This software is supplied subject our standard software license agreement.
;A non-binding summary is as follows:
;"You may use the software in your own code at your own risk.
;" ADI inc disclaims any warranty or liability of its use".
;The full standard software license agreement is available on request and
; remains the binding agreement.

#include        <ADuC7030.inc>

; UrtCfg==========Configures UART.
; C Function prototype: int UrtCfg(int iBaud, int iFormat);
;iBaud:{URT_6,URT_12,URT_24,URT_48,URT_96,URT_192,URT_384,URT_576,URT_1152,URT_2304,URT_4608}
;iFormat:{URT_68|URT_78|URT_2S|URT_PE|URT_PODD|URT_PS|URT_BR}
; Description of Function: Configure the UART
; User interface:
;   Set iBaud to the baudrate required:
;       Either set iBaud to one of 0 to 10 for one of the
;        baudrates listed above (URT_nx100). In this case
;        the clock must be 20480000Hz+-1%.
;       Alternatively set iBaud to (Urt/Baudrate)*2048
;        (e.g. (20480000/9600)*2048 = 4369066).
;   Set iFormat to the data format required (COMCON0):
;       COMCON0.0 = URT_68\_/ Data length 5, 6, 7 or 8 bits.
;       COMCON0.1 = URT_78/ \
;       COMCON0.2 = URT_STP2 -> 2 stop bits.
;       COMCON0.3 = URT_PE -> Parity used.
;       COMCON0.4 = URT_PODD -> Odd parity.
;       COMCON0.5 = URT_PS -> Sticky parity.
;       COMCON0.6 = URT_BR -> Break (Set SOUT to 0).
;       COMCON0.7 to COMCON.31 -> Ignored.
;   Call UrtCfg.
;   Returns 1 if successful or 0 otherwise
; Robustness:
;   User must check that rounding errors plus clock errors
;    are acceptable.
;   The baudrate is set using the current CD bits. The baudrate
;   will change if the CD bits are changed subsequently.
; Side effects:
;   Corrupts r1, r2 and r3.
;   Reconfigures the UART.

                RSEG CODE
                MODULE UrtCfg
                PUBLIC UrtCfg
                CODE32

UrtCfg:
        ldr     r3,Z                    ; r3 = Z_BASE;
        ldr     r2,[r3,#Z_GP1CON]       ; GP1CON &= 0xffffff00;
        and     r2,r2,#0xffffff00
        orr     r2,r2,#0x00000011       ; GP1CON |= 0x00000011;
        str     r2,[r3,#Z_GP1CON]       ;
UrtBr:  cmp     r0,#10                  ;if(iBaud>10)
        bls     UrtLst
        mov     r0,r0,LSR#5             ; r0 = iBaud>>5;
        b       UrtDl
UrtLst: cmp     r0,#6                   ;else { if(iBaud<6)
        ldrls   r2,=0x215555            ;r2 = 2184533; //(20480000*2048)/(600*16*2)
        ldrhi   r2,=0x2C71C7            ; else r2 = 2912711; } //(20480000*128*2048)/(57600*16*2)
        mov     r0,r2,LSR r0            ;r0 = r2>>iBaud;
UrtDl:
        mov     r2,#0                   ;COMDIV2 = 0;
        str     r2,[r3,#Z_COMIEN0]      ;COMIEN0 = 0;
        ldr     r2,[r3,#Z_POWCON]       ;r0 >>= CD;
        and     r2,r2,#7
        mov     r0,r0,LSR r2

        mov     r3,r0
        mov     r0,r3,LSR #11          ; r0 = r3 / 2048

; Calculate the value to write to COMDIV2
; To allow division by shifting (closest approximation) calculate the closest
; 2^x that is below the DL value
        mov     r2,#0x0
LpDL:   add     r2,r2,#1
        movs    r0,r0,LSR #1
        bne     LpDL
        sub     r2,r2,#1

;       Use adjusted DL for the N value that we will calculate now
        mov     r0,#1
        mov     r0,r0,LSL r2

; Calculate N
        mov     r2,r3,LSR r2
        sub     r2,r2,#0x800
        orr     r2,r2,#0x8800

       ; mov     r0,r3,LSR #11           ; r0 = r3 / 2048
        ldr     r3,Z                    ; r3 = Z_BASE;
        str     r2,[r3,#Z_COMDIV2]

        mov     r2,#0x83                ;COMDIV0-1 = r0.
        strb    r2,[r3,#Z_COMCON0]
        mov     r2,#0x80                ;COMCON1   = 0x80
        strb    r2,[r3,#Z_COMCON1]
        strb    r0,[r3,#Z_COMDIV0]
        mov     r0,r0,LSR#8
        strb    r0,[r3,#Z_COMDIV1]
        strb    r1,[r3,#Z_COMCON0]
        moveq   r0,#0                   ;r0 = 0;
        bx      lr
Z:
  DC32   Z_BASE
                 ENDMOD  UrtCfg

                 END

